@@ -7,6 +7,7 @@
 #include <lxc/attach_options.h>
 #include <sys/types.h>
 
+#include "compiler.h"
 #include "namespace.h"
 
 struct lxc_conf;
@@ -20,10 +21,10 @@ struct lxc_proc_context_info {
 	int ns_fd[LXC_NS_MAX];
 };
 
-extern int lxc_attach(struct lxc_container *container,
-		      lxc_attach_exec_t exec_function, void *exec_payload,
-		      lxc_attach_options_t *options, pid_t *attached_process);
+__hidden extern int lxc_attach(struct lxc_container *container, lxc_attach_exec_t exec_function,
+			       void *exec_payload, lxc_attach_options_t *options,
+			       pid_t *attached_process);
 
-extern int lxc_attach_remount_sys_proc(void);
+__hidden extern int lxc_attach_remount_sys_proc(void);
 
 #endif /* __LXC_ATTACH_H */
