@@ -27,6 +27,18 @@
 
 lxc_log_define(cgroup2_devices, cgroup);
 
+#ifndef BPF_LOG_LEVEL1
+#define BPF_LOG_LEVEL1 1
+#endif
+
+#ifndef BPF_LOG_LEVEL2
+#define BPF_LOG_LEVEL2 2
+#endif
+
+#ifndef BPF_LOG_LEVEL
+#define BPF_LOG_LEVEL (BPF_LOG_LEVEL1 | BPF_LOG_LEVEL2)
+#endif
+
 static int bpf_program_add_instructions(struct bpf_program *prog,
 					const struct bpf_insn *instructions,
 					size_t count)
@@ -118,29 +130,28 @@ void bpf_program_free(struct bpf_program *prog)
 			   .off = 0,                   \
 			   .imm = 0})
 
-static int bpf_access_mask(const char *acc)
+static int bpf_access_mask(const char *acc, __u32 *mask)
 {
-	int mask = 0;
-
 	if (!acc)
-		return mask;
+		return 0;
 
-	for (; *acc; acc++)
+	for (; *acc; acc++) {
 		switch (*acc) {
 		case 'r':
-			mask |= BPF_DEVCG_ACC_READ;
+			*mask |= BPF_DEVCG_ACC_READ;
 			break;
 		case 'w':
-			mask |= BPF_DEVCG_ACC_WRITE;
+			*mask |= BPF_DEVCG_ACC_WRITE;
 			break;
 		case 'm':
-			mask |= BPF_DEVCG_ACC_MKNOD;
+			*mask |= BPF_DEVCG_ACC_MKNOD;
 			break;
 		default:
 			return -EINVAL;
 		}
+	}
 
-	return mask;
+	return 0;
 }
 
 static int bpf_device_type(char type)
@@ -157,10 +168,9 @@ static int bpf_device_type(char type)
 	return -1;
 }
 
-static inline bool bpf_device_all_access(int access_mask)
+static inline bool bpf_device_all_access(__u32 access_mask)
 {
-	return (access_mask == (BPF_DEVCG_ACC_READ | BPF_DEVCG_ACC_WRITE |
-				BPF_DEVCG_ACC_MKNOD));
+	return access_mask == (BPF_DEVCG_ACC_READ | BPF_DEVCG_ACC_WRITE | BPF_DEVCG_ACC_MKNOD);
 }
 
 struct bpf_program *bpf_program_new(uint32_t prog_type)
@@ -174,9 +184,9 @@ struct bpf_program *bpf_program_new(uint32_t prog_type)
 	prog->prog_type = prog_type;
 	prog->kernel_fd = -EBADF;
 	/*
-	 * By default a whitelist is used unless the user tells us otherwise.
+	 * By default a allowlist is used unless the user tells us otherwise.
 	 */
-	prog->device_list_type = LXC_BPF_DEVICE_CGROUP_WHITELIST;
+	prog->device_list_type = LXC_BPF_DEVICE_CGROUP_ALLOWLIST;
 
 	return move_ptr(prog);
 }
@@ -208,7 +218,8 @@ int bpf_program_init(struct bpf_program *prog)
 int bpf_program_append_device(struct bpf_program *prog, struct device_item *device)
 {
 	int jump_nr = 1;
-	int access_mask, device_type, ret;
+	__u32 access_mask = 0;
+	int device_type, ret;
 	struct bpf_insn bpf_access_decision[2];
 
 	if (!prog || !device)
@@ -220,6 +231,13 @@ int bpf_program_append_device(struct bpf_program *prog, struct device_item *devi
 		return 0;
 	}
 
+	ret = bpf_access_mask(device->access, &access_mask);
+	if (ret < 0)
+		return log_error_errno(ret, -ret, "Invalid access mask specified %s", device->access);
+
+	if (!bpf_device_all_access(access_mask))
+		jump_nr++;
+
 	device_type = bpf_device_type(device->type);
 	if (device_type < 0)
 		return log_error_errno(-1, EINVAL, "Invalid bpf cgroup device type %c", device->type);
@@ -227,19 +245,17 @@ int bpf_program_append_device(struct bpf_program *prog, struct device_item *devi
 	if (device_type > 0)
 		jump_nr++;
 
-	access_mask = bpf_access_mask(device->access);
-	if (!bpf_device_all_access(access_mask))
-		jump_nr += 3;
-
 	if (device->major != -1)
 		jump_nr++;
 
 	if (device->minor != -1)
 		jump_nr++;
 
-	if (device_type > 0) {
+	if (!bpf_device_all_access(access_mask)) {
 		struct bpf_insn ins[] = {
-		    BPF_JMP_IMM(BPF_JNE, BPF_REG_2, device_type, jump_nr--),
+		    BPF_MOV32_REG(BPF_REG_1, BPF_REG_3),
+		    BPF_ALU32_IMM(BPF_AND, BPF_REG_1, access_mask),
+		    BPF_JMP_REG(BPF_JNE, BPF_REG_1, BPF_REG_3, jump_nr--),
 		};
 
 		ret = bpf_program_add_instructions(prog, ins, ARRAY_SIZE(ins));
@@ -247,14 +263,11 @@ int bpf_program_append_device(struct bpf_program *prog, struct device_item *devi
 			return log_error_errno(-1, errno, "Failed to add instructions to bpf cgroup program");
 	}
 
-	if (!bpf_device_all_access(access_mask)) {
+	if (device_type > 0) {
 		struct bpf_insn ins[] = {
-		    BPF_MOV32_REG(BPF_REG_1, BPF_REG_3),
-		    BPF_ALU32_IMM(BPF_AND, BPF_REG_1, access_mask),
-		    BPF_JMP_REG(BPF_JNE, BPF_REG_1, BPF_REG_3, jump_nr),
+		    BPF_JMP_IMM(BPF_JNE, BPF_REG_2, device_type, jump_nr--),
 		};
 
-		jump_nr -= 3;
 		ret = bpf_program_add_instructions(prog, ins, ARRAY_SIZE(ins));
 		if (ret)
 			return log_error_errno(-1, errno, "Failed to add instructions to bpf cgroup program");
@@ -283,7 +296,7 @@ int bpf_program_append_device(struct bpf_program *prog, struct device_item *devi
 	bpf_access_decision[0] = BPF_MOV64_IMM(BPF_REG_0, device->allow);
 	bpf_access_decision[1] = BPF_EXIT_INSN();
 	ret = bpf_program_add_instructions(prog, bpf_access_decision,
-					    ARRAY_SIZE(bpf_access_decision));
+					   ARRAY_SIZE(bpf_access_decision));
 	if (ret)
 		return log_error_errno(-1, errno, "Failed to add instructions to bpf cgroup program");
 
@@ -298,9 +311,9 @@ int bpf_program_finalize(struct bpf_program *prog)
 		return ret_set_errno(-1, EINVAL);
 
 	TRACE("Implementing %s bpf device cgroup program",
-	      prog->device_list_type == LXC_BPF_DEVICE_CGROUP_BLACKLIST
-		  ? "blacklist"
-		  : "whitelist");
+	      prog->device_list_type == LXC_BPF_DEVICE_CGROUP_DENYLIST
+		  ? "denylist"
+		  : "allowlist");
 
 	ins[0] = BPF_MOV64_IMM(BPF_REG_0, prog->device_list_type);
 	ins[1] = BPF_EXIT_INSN();
@@ -308,29 +321,34 @@ int bpf_program_finalize(struct bpf_program *prog)
 }
 
 static int bpf_program_load_kernel(struct bpf_program *prog, char *log_buf,
-				   size_t log_size)
+				   __u32 log_size, __u32 log_level)
 {
 	union bpf_attr attr;
 
+	if ((log_size != 0 && !log_buf) || (log_size == 0 && log_buf))
+		return ret_errno(EINVAL);
+
 	if (prog->kernel_fd >= 0) {
 		memset(log_buf, 0, log_size);
 		return 0;
 	}
 
 	attr = (union bpf_attr){
-	    .prog_type	= prog->prog_type,
-	    .insns	= PTR_TO_UINT64(prog->instructions),
-	    .insn_cnt	= prog->n_instructions,
-	    .license	= PTR_TO_UINT64("GPL"),
-	    .log_buf	= PTR_TO_UINT64(log_buf),
-	    .log_level	= !!log_buf,
-	    .log_size	= log_size,
+		.prog_type	= prog->prog_type,
+		.insns		= PTR_TO_UINT64(prog->instructions),
+		.insn_cnt	= prog->n_instructions,
+		.license	= PTR_TO_UINT64("GPL"),
+		.log_buf	= PTR_TO_UINT64(log_buf),
+		.log_level	= log_level,
+		.log_size	= log_size,
 	};
 
 	prog->kernel_fd = bpf(BPF_PROG_LOAD, &attr, sizeof(attr));
 	if (prog->kernel_fd < 0)
-		return log_error_errno(-1, errno, "Failed to load bpf program");
+		return log_error_errno(-1, errno, "Failed to load bpf program: %s",
+				       log_buf ?: "(null)");
 
+	TRACE("Loaded bpf program: %s", log_buf ?: "(null)");
 	return 0;
 }
 
@@ -359,7 +377,7 @@ int bpf_program_cgroup_attach(struct bpf_program *prog, int type,
 			return true;
 	}
 
-	ret = bpf_program_load_kernel(prog, NULL, 0);
+	ret = bpf_program_load_kernel(prog, NULL, 0, 0);
 	if (ret < 0)
 		return log_error_errno(-1, ret, "Failed to load bpf program");
 
@@ -450,12 +468,12 @@ int bpf_list_add_device(struct lxc_conf *conf, struct device_item *device)
 		if (cur->global_rule > LXC_BPF_DEVICE_CGROUP_LOCAL_RULE &&
 		    device->global_rule > LXC_BPF_DEVICE_CGROUP_LOCAL_RULE) {
 			TRACE("Switched from %s to %s",
-			      cur->global_rule == LXC_BPF_DEVICE_CGROUP_WHITELIST
-				  ? "whitelist"
-				  : "blacklist",
-			      device->global_rule == LXC_BPF_DEVICE_CGROUP_WHITELIST
-				  ? "whitelist"
-				  : "blacklist");
+			      cur->global_rule == LXC_BPF_DEVICE_CGROUP_ALLOWLIST
+				  ? "allowlist"
+				  : "denylist",
+			      device->global_rule == LXC_BPF_DEVICE_CGROUP_ALLOWLIST
+				  ? "allowlist"
+				  : "denylist");
 			cur->global_rule = device->global_rule;
 			return 1;
 		}
@@ -521,7 +539,7 @@ bool bpf_devices_cgroup_supported(void)
 	if (ret < 0)
 		return log_trace(false, "Failed to add new instructions to bpf device cgroup program");
 
-	ret = bpf_program_load_kernel(prog, NULL, 0);
+	ret = bpf_program_load_kernel(prog, NULL, 0, 0);
 	if (ret < 0)
 		return log_trace(false, "Failed to load new bpf device cgroup program");
 
