@@ -1,23 +1,8 @@
-/* liblxcapi
- *
- * Copyright © 2017 Christian Brauner <christian.brauner@ubuntu.com>.
- * Copyright © 2017 Canonical Ltd.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2, as
- * published by the Free Software Foundation.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+/* SPDX-License-Identifier: LGPL-2.1+ */
 
-#define _GNU_SOURCE
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE 1
+#endif
 #define __STDC_FORMAT_MACROS
 #include <errno.h>
 #include <inttypes.h>
@@ -25,17 +10,19 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
 #include <sys/mman.h>
+#include <unistd.h>
 
+#include "config.h"
 #include "ringbuf.h"
+#include "syscall_wrappers.h"
 #include "utils.h"
 
 int lxc_ringbuf_create(struct lxc_ringbuf *buf, size_t size)
 {
+	__do_close int memfd = -EBADF;
 	char *tmp;
 	int ret;
-	int memfd = -1;
 
 	buf->size = size;
 	buf->r_off = 0;
@@ -76,14 +63,10 @@ int lxc_ringbuf_create(struct lxc_ringbuf *buf, size_t size)
 	if (tmp == MAP_FAILED || tmp != (buf->addr + buf->size))
 		goto on_error;
 
-	close(memfd);
-
 	return 0;
 
 on_error:
 	lxc_ringbuf_release(buf);
-	if (memfd >= 0)
-		close(memfd);
 	return -1;
 }
 
