@@ -1,40 +1,28 @@
-/*
- *
- * Copyright © 2013 Serge Hallyn <serge.hallyn@ubuntu.com>.
- * Copyright © 2013 Canonical Ltd.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License version 2, as
- * published by the Free Software Foundation.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- */
+/* SPDX-License-Identifier: GPL-2.0-only */
 
-#define _GNU_SOURCE
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE 1
+#endif
 #include <fcntl.h>
 #include <libgen.h>
 #include <stdio.h>
 #include <string.h>
-#include <unistd.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <unistd.h>
 
 #include <lxc/lxccontainer.h>
 
 #include "arguments.h"
+#include "config.h"
 #include "log.h"
 #include "utils.h"
 
 lxc_log_define(lxc_destroy, lxc);
 
 static int my_parser(struct lxc_arguments *args, int c, char *arg);
+static bool do_destroy(struct lxc_container *c);
+static bool do_destroy_with_snapshots(struct lxc_container *c);
 
 static const struct option my_longopts[] = {
 	{"force", no_argument, 0, 'f'},
@@ -43,8 +31,8 @@ static const struct option my_longopts[] = {
 };
 
 static struct lxc_arguments my_args = {
-	.progname = "lxc-destroy",
-	.help     = "\
+	.progname     = "lxc-destroy",
+	.help         = "\
 --name=NAME [-f] [-P lxcpath]\n\
 \n\
 lxc-destroy destroys a container with the identifier NAME\n\
@@ -54,107 +42,36 @@ Options :\n\
   -s, --snapshots   destroy including all snapshots\n\
   -f, --force       wait for the container to shut down\n\
   --rcfile=FILE     Load configuration file FILE\n",
-	.options  = my_longopts,
-	.parser   = my_parser,
-	.checker  = NULL,
-	.task     = DESTROY,
+	.options      = my_longopts,
+	.parser       = my_parser,
+	.checker      = NULL,
+	.log_priority = "ERROR",
+	.log_file     = "none",
+	.task         = DESTROY,
 };
 
-static bool do_destroy(struct lxc_container *c);
-static bool do_destroy_with_snapshots(struct lxc_container *c);
-
-int main(int argc, char *argv[])
-{
-	struct lxc_container *c;
-	struct lxc_log log;
-	bool bret;
-
-	if (lxc_arguments_parse(&my_args, argc, argv))
-		exit(EXIT_FAILURE);
-
-	/* Only create log if explicitly instructed */
-	if (my_args.log_file || my_args.log_priority) {
-		log.name = my_args.name;
-		log.file = my_args.log_file;
-		log.level = my_args.log_priority;
-		log.prefix = my_args.progname;
-		log.quiet = my_args.quiet;
-		log.lxcpath = my_args.lxcpath[0];
-
-		if (lxc_log_init(&log))
-			exit(EXIT_FAILURE);
-	}
-
-	c = lxc_container_new(my_args.name, my_args.lxcpath[0]);
-	if (!c) {
-		ERROR("System error loading container");
-		exit(EXIT_FAILURE);
-	}
-
-	if (my_args.rcfile) {
-		c->clear_config(c);
-
-		if (!c->load_config(c, my_args.rcfile)) {
-			ERROR("Failed to load rcfile");
-			lxc_container_put(c);
-			exit(EXIT_FAILURE);
-		}
-
-		c->configfile = strdup(my_args.rcfile);
-		if (!c->configfile) {
-			ERROR("Out of memory setting new config filename");
-			lxc_container_put(c);
-			exit(EXIT_FAILURE);
-		}
-	}
-
-	if (!c->may_control(c)) {
-		ERROR("Insufficent privileges to control %s", my_args.name);
-		lxc_container_put(c);
-		exit(EXIT_FAILURE);
-	}
-
-	if (!c->is_defined(c)) {
-		ERROR("Container is not defined");
-		lxc_container_put(c);
-		exit(EXIT_FAILURE);
-	}
-
-	if (my_args.task == SNAP) {
-		bret = do_destroy_with_snapshots(c);
-		if (bret)
-			ERROR("Destroyed container %s including snapshots", my_args.name);
-	} else {
-		bret = do_destroy(c);
-		if (bret)
-			ERROR("Destroyed container %s", my_args.name);
-	}
-
-	lxc_container_put(c);
-
-	if (bret)
-		exit(EXIT_SUCCESS);
-
-	exit(EXIT_FAILURE);
-}
-
 static int my_parser(struct lxc_arguments *args, int c, char *arg)
 {
 	switch (c) {
-	case 'f': args->force = 1; break;
-	case 's': args->task = SNAP; break;
+	case 'f':
+		args->force = 1;
+		break;
+	case 's':
+		args->task = SNAP;
+		break;
 	}
 	return 0;
 }
 
 static bool do_destroy(struct lxc_container *c)
 {
+	int ret;
 	bool bret = true;
-	char path[MAXPATHLEN];
+	char path[PATH_MAX];
 
 	/* First check whether the container has dependent clones or snapshots. */
-	int ret = snprintf(path, MAXPATHLEN, "%s/%s/lxc_snapshots", c->config_path, c->name);
-	if (ret < 0 || ret >= MAXPATHLEN)
+	ret = snprintf(path, PATH_MAX, "%s/%s/lxc_snapshots", c->config_path, c->name);
+	if (ret < 0 || ret >= PATH_MAX)
 		return false;
 
 	if (file_exists(path)) {
@@ -162,8 +79,8 @@ static bool do_destroy(struct lxc_container *c)
 		return false;
 	}
 
-	ret = snprintf(path, MAXPATHLEN, "%s/%s/snaps", c->config_path, c->name);
-	if (ret < 0 || ret >= MAXPATHLEN)
+	ret = snprintf(path, PATH_MAX, "%s/%s/snaps", c->config_path, c->name);
+	if (ret < 0 || ret >= PATH_MAX)
 		return false;
 
 	if (rmdir(path) < 0 && errno != ENOENT) {
@@ -187,9 +104,8 @@ static bool do_destroy(struct lxc_container *c)
 		char buf[256];
 
 		ret = c->get_config_item(c, "lxc.ephemeral", buf, 256);
-		if (ret > 0 && strcmp(buf, "0") == 0) {
+		if (ret > 0 && strcmp(buf, "0") == 0)
 			bret = c->destroy(c);
-		}
 	}
 
 	if (!bret) {
@@ -205,7 +121,7 @@ static bool do_destroy_with_snapshots(struct lxc_container *c)
 	struct lxc_container *c1;
 	struct stat fbuf;
 	bool bret = false;
-	char path[MAXPATHLEN];
+	char path[PATH_MAX];
 	char *buf = NULL;
 	char *lxcpath = NULL;
 	char *lxcname = NULL;
@@ -214,8 +130,8 @@ static bool do_destroy_with_snapshots(struct lxc_container *c)
 	ssize_t bytes;
 
 	/* Destroy clones. */
-	ret = snprintf(path, MAXPATHLEN, "%s/%s/lxc_snapshots", c->config_path, c->name);
-	if (ret < 0 || ret >= MAXPATHLEN)
+	ret = snprintf(path, PATH_MAX, "%s/%s/lxc_snapshots", c->config_path, c->name);
+	if (ret < 0 || ret >= PATH_MAX)
 		return false;
 
 	fd = open(path, O_RDONLY | O_CLOEXEC);
@@ -262,8 +178,8 @@ static bool do_destroy_with_snapshots(struct lxc_container *c)
 	}
 
 	/* Destroy snapshots located in the containers snap/ folder. */
-	ret = snprintf(path, MAXPATHLEN, "%s/%s/snaps", c->config_path, c->name);
-	if (ret < 0 || ret >= MAXPATHLEN)
+	ret = snprintf(path, PATH_MAX, "%s/%s/snaps", c->config_path, c->name);
+	if (ret < 0 || ret >= PATH_MAX)
 		return false;
 
 	if (rmdir(path) < 0 && errno != ENOENT)
@@ -273,3 +189,75 @@ static bool do_destroy_with_snapshots(struct lxc_container *c)
 
 	return bret;
 }
+
+int main(int argc, char *argv[])
+{
+	struct lxc_container *c;
+	struct lxc_log log;
+	bool bret;
+
+	if (lxc_arguments_parse(&my_args, argc, argv))
+		exit(EXIT_FAILURE);
+
+	log.name = my_args.name;
+	log.file = my_args.log_file;
+	log.level = my_args.log_priority;
+	log.prefix = my_args.progname;
+	log.quiet = my_args.quiet;
+	log.lxcpath = my_args.lxcpath[0];
+
+	if (lxc_log_init(&log))
+		exit(EXIT_FAILURE);
+
+	c = lxc_container_new(my_args.name, my_args.lxcpath[0]);
+	if (!c) {
+		ERROR("System error loading container");
+		exit(EXIT_FAILURE);
+	}
+
+	if (my_args.rcfile) {
+		c->clear_config(c);
+
+		if (!c->load_config(c, my_args.rcfile)) {
+			ERROR("Failed to load rcfile");
+			lxc_container_put(c);
+			exit(EXIT_FAILURE);
+		}
+
+		c->configfile = strdup(my_args.rcfile);
+		if (!c->configfile) {
+			ERROR("Out of memory setting new config filename");
+			lxc_container_put(c);
+			exit(EXIT_FAILURE);
+		}
+	}
+
+	if (!c->may_control(c)) {
+		ERROR("Insufficent privileges to control %s", my_args.name);
+		lxc_container_put(c);
+		exit(EXIT_FAILURE);
+	}
+
+	if (!c->is_defined(c)) {
+		ERROR("Container is not defined");
+		lxc_container_put(c);
+		exit(EXIT_FAILURE);
+	}
+
+	if (my_args.task == SNAP) {
+		bret = do_destroy_with_snapshots(c);
+		if (bret)
+			INFO("Destroyed container %s including snapshots", my_args.name);
+	} else {
+		bret = do_destroy(c);
+		if (bret)
+			INFO("Destroyed container %s", my_args.name);
+	}
+
+	lxc_container_put(c);
+
+	if (bret)
+		exit(EXIT_SUCCESS);
+
+	exit(EXIT_FAILURE);
+}
